package cn.com.infosec.mobile.android.framework.utils;

import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.ASN1Set;
import cn.com.infosec.asn1.DERBitString;
import cn.com.infosec.asn1.DERBoolean;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.DEROctetString;
import cn.com.infosec.asn1.DEROutputStream;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.asn1.cms.Attribute;
import cn.com.infosec.asn1.pkcs.CertificationRequest;
import cn.com.infosec.asn1.pkcs.CertificationRequestInfo;
import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.x509.AlgorithmIdentifier;
import cn.com.infosec.asn1.x509.SubjectPublicKeyInfo;
import cn.com.infosec.asn1.x509.X509Extension;
import cn.com.infosec.asn1.x509.X509Name;
import cn.com.infosec.mobile.android.framework.crypto.HSM;
import cn.com.infosec.mobile.android.framework.crypto.IHSM;
import cn.com.infosec.mobile.android.framework.crypto.SM2Id;
import cn.com.infosec.util.Base64;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.util.Enumeration;
import java.util.Hashtable;

public class Pkcs10 {
   private static final String CERTIFICATE_EXT_OID = "1.3.6.1.4.1.311.2.1.14";
   private static final String CERTIFICATE_REQ_BEGIN = "-----BEGIN NEW CERTIFICATE REQUEST-----";
   private static final String CERTIFICATE_REQ_END = "-----END NEW CERTIFICATE REQUEST-----";
   private CertificationRequest csr;
   private X509Name subject;
   private PublicKey pk;

   public static Pkcs10 parseCSR(byte[] p10data) {
      return parseCSR(p10data, true);
   }

   public static Pkcs10 parseCSR(byte[] p10data, boolean verifySignature) {
      boolean var10000 = verifySignature;
      ByteArrayInputStream verifySignature1;
      ByteArrayInputStream var10001 = verifySignature1 = new ByteArrayInputStream;
      var10001.<init>(p10data);
      ASN1InputStream p10data1;
      p10data1 = new ASN1InputStream.<init>(verifySignature1);
      var10001.close();
      ASN1Sequence p10data2 = (ASN1Sequence)p10data1.readObject();
      CertificationRequest verifySignature2;
      CertificationRequest var17 = verifySignature2 = new CertificationRequest;
      var17.<init>(p10data2);
      CertificationRequestInfo p10data3;
      PublicKey var2 = genPublicKey((p10data3 = var17.getCertificationRequestInfo()).getSubjectPublicKeyInfo());
      Pkcs10 var3;
      Pkcs10 var14;
      if (var10000) {
         byte[] var11 = verifySignature2.getSignature().getBytes();
         String var4 = verifySignature2.getSignatureAlgorithm().getObjectId().getId();
         ByteArrayOutputStream var5;
         ByteArrayOutputStream var15 = var5 = new ByteArrayOutputStream;
         var15.<init>();
         DEROutputStream var10002 = new DEROutputStream(var5);
         var10002.writeObject(p10data3.getDERObject());
         var10002.close();
         byte[] var12 = var15.toByteArray();
         var15.close();
         IHSM var16 = HSM.getInst("");
         byte[] var18 = var12;
         String var13 = SM2Id.getVerifyId("CSR");
         if (var16.verify(var18, var11, var2, var4, var13)) {
            var14 = var3 = new Pkcs10;
            var3.<init>();
            var3.csr = verifySignature2;
            var3.pk = var2;
            var14.subject = p10data3.getSubject();
            return var14;
         } else {
            throw new Exception("Invalid CSR: verify signature failed.");
         }
      } else {
         var14 = var3 = new Pkcs10;
         var3.<init>();
         var3.csr = verifySignature2;
         var3.pk = var2;
         var14.subject = p10data3.getSubject();
         return var14;
      }
   }

   public static Pkcs10 parseMozillaCSR(byte[] p10data) {
      ASN1Sequence p10data1;
      if ((p10data1 = (ASN1Sequence)(new ASN1InputStream(new ByteArrayInputStream(p10data))).readObject()).size() == 3) {
         ASN1Sequence var1;
         if ((var1 = (ASN1Sequence)p10data1.getObjectAt(0)).size() == 2) {
            ASN1Sequence var10000 = p10data1;
            ASN1Sequence var10001 = p10data1;
            PublicKey p10data2 = genPublicKey(new SubjectPublicKeyInfo((ASN1Sequence)var1.getObjectAt(0)));
            String var2 = AlgorithmIdentifier.getInstance((DERSequence)var10001.getObjectAt(1)).getObjectId().getId();
            byte[] var3 = ((DERBitString)var10000.getObjectAt(2)).getBytes();
            if (HSM.getInst("").verify(var1.getEncoded(), var3, p10data2, var2, (String)null)) {
               Pkcs10 var6 = new Pkcs10();
               var6.pk = p10data2;
               return var6;
            } else {
               throw new Exception("Invalid CSR: verify signature failed.");
            }
         } else {
            throw new IllegalArgumentException("invalid PKAC (len): " + var1.size());
         }
      } else {
         throw new IllegalArgumentException("invalid SPKAC (size):" + p10data1.size());
      }
   }

   public static String genCSR(X509Name dn, String keyIdx, int keyLen, String signAlg, char[] pwd, IHSM hsm, String sm2Id) {
      String var7;
      String var8;
      if (signAlg.toLowerCase().indexOf("rsa") > -1) {
         var7 = "RSA";
         if (signAlg.equalsIgnoreCase("SHA1withRSA")) {
            var8 = PKCSObjectIdentifiers.sha1WithRSAEncryption.getId();
         } else {
            if (!signAlg.equalsIgnoreCase("SHA256withRSA")) {
               throw new Exception(signAlg + " NOT support.");
            }

            var8 = PKCSObjectIdentifiers.sha256WithRSAEncryption.getId();
         }
      } else {
         var7 = "SM2";
         var8 = "1.2.156.10197.1.501";
      }

      PublicKey keyIdx1 = hsm.exportPublicKey(var7, "SIGN", keyIdx, keyLen, pwd);
      SubjectPublicKeyInfo hsm1;
      hsm1 = new SubjectPublicKeyInfo.<init>((ASN1Sequence)(new ASN1InputStream(new ByteArrayInputStream(keyIdx1.getEncoded()))).readObject());
      CertificationRequestInfo keyIdx2;
      keyIdx2 = new CertificationRequestInfo.<init>(dn, hsm1, (ASN1Set)null);
      ByteArrayOutputStream dn1;
      ByteArrayOutputStream var10002 = dn1 = new ByteArrayOutputStream;
      var10002.<init>();
      DEROutputStream var10004 = new DEROutputStream(dn1);
      var10004.writeObject(keyIdx2.getDERObject());
      var10004.close();
      byte[] dn2 = var10002.toByteArray();
      var10002.close();
      dn2 = hsm.sign(keyIdx, keyLen, pwd, dn2, signAlg, sm2Id);
      DERBitString keyLen1;
      keyLen1 = new DERBitString.<init>(dn2);
      CertificationRequest dn3;
      CertificationRequest var21 = dn3 = new CertificationRequest;
      AlgorithmIdentifier keyIdx3;
      keyIdx3 = new AlgorithmIdentifier.<init>(var8);
      var21.<init>(keyIdx2, keyIdx3, keyLen1);
      ByteArrayOutputStream keyIdx4;
      ByteArrayOutputStream var22 = keyIdx4 = new ByteArrayOutputStream;
      var22.<init>();
      DEROutputStream var26 = new DEROutputStream(keyIdx4);
      var26.writeObject(dn3.getDERObject());
      var26.close();
      byte[] var23 = var22.toByteArray();
      keyIdx4.close();
      String dn4 = Base64.encode(var23);
      keyIdx = System.getProperty("line.separator");
      StringBuffer keyLen2;
      StringBuffer var25 = keyLen2 = new StringBuffer;
      var25.<init>();
      var25.append("-----BEGIN NEW CERTIFICATE REQUEST-----").append(keyIdx);
      int signAlg1 = 0;

      while(signAlg1 < dn4.length()) {
         keyLen2.append(dn4.charAt(signAlg1));
         if (++signAlg1 % 64 == 0) {
            keyLen2.append(keyIdx);
         }
      }

      keyLen2.append(keyIdx).append("-----END NEW CERTIFICATE REQUEST-----");
      return keyLen2.toString();
   }

   private static PublicKey genPublicKey(SubjectPublicKeyInfo kinfo) {
      String var1;
      String var10000 = var1 = kinfo.getAlgorithmId().getObjectId().getId();
      X509EncodedKeySpec var2;
      var2 = new X509EncodedKeySpec.<init>(kinfo.getEncoded());
      KeyFactory var3;
      if (var10000.equalsIgnoreCase("1.2.840.10045.2.1")) {
         var3 = KeyFactory.getInstance("EC", "INFOSEC");
      } else {
         if (!var1.equalsIgnoreCase(PKCSObjectIdentifiers.rsaEncryption.getId())) {
            throw new NoSuchAlgorithmException("Public Key getAlgorithmId not support (" + var1 + ").");
         }

         var3 = KeyFactory.getInstance("RSA", "INFOSEC");
      }

      return var3.generatePublic(var2);
   }

   public PublicKey getPublicKey() {
      return this.pk;
   }

   public X509Name getSubject() {
      return this.subject;
   }

   public Hashtable<DERObjectIdentifier, X509Extension> getCertificateExtension() {
      Hashtable var10;
      var10 = new Hashtable.<init>();
      CertificationRequest var1;
      if ((var1 = this.csr) == null) {
         return var10;
      } else {
         ASN1Set var11;
         if ((var11 = var1.getCertificationRequestInfo().getAttributes()) == null) {
            return var10;
         } else {
            int var2 = var11.size();

            for(int var3 = 0; var3 < var2; ++var3) {
               Attribute var4;
               if ((var4 = Attribute.getInstance(var11.getObjectAt(var3))).getAttrType().getId().equals("1.3.6.1.4.1.311.2.1.14")) {
                  Enumeration var12 = ((ASN1Sequence)var4.getAttrValues().getObjectAt(0)).getObjects();

                  while(var12.hasMoreElements()) {
                     Object var5;
                     if ((var5 = var12.nextElement()) instanceof ASN1Sequence) {
                        ASN1Sequence var15 = (ASN1Sequence)var5;
                        ASN1Sequence var13;
                        int var6 = (var13 = (ASN1Sequence)var5).size();
                        boolean var7 = false;
                        DEROctetString var8 = null;
                        if (!(var15.getObjectAt(0) instanceof DERObjectIdentifier)) {
                           throw new IOException("format error, not a oid");
                        }

                        DERObjectIdentifier var9 = (DERObjectIdentifier)var13.getObjectAt(0);
                        if (var6 != 3 && var6 != 2) {
                           throw new IOException("format error, not an Extension");
                        }

                        if (var6 == 2) {
                           var7 = false;
                           if (!(var13.getObjectAt(1) instanceof DEROctetString)) {
                              throw new IOException("format error, not a DEROctetString");
                           }

                           var8 = (DEROctetString)var13.getObjectAt(1);
                        } else if (var6 == 3) {
                           if (!(var13.getObjectAt(1) instanceof DERBoolean)) {
                              throw new IOException("format error, not a DERBoolean");
                           }

                           var7 = ((DERBoolean)var13.getObjectAt(1)).isTrue();
                           if (!(var13.getObjectAt(2) instanceof DEROctetString)) {
                              throw new IOException("format error, not a DEROctetString");
                           }

                           var8 = (DEROctetString)var13.getObjectAt(2);
                        }

                        X509Extension var14;
                        var14 = new X509Extension.<init>(var7, var8);
                        if (var9 != null) {
                           var10.put(var9, var14);
                        }
                     }
                  }
               }
            }

            return var10;
         }
      }
   }

   public CertificationRequest getCsr() {
      return this.csr;
   }
}
